import { Chess } from "../chessops/src/chess";
import { Move } from "../chessops/src/types";
import { chessgroundDests } from "../chessops/src/compat";
export { chessgroundDests };
export declare type Uci = string;
export declare type San = string;
export declare type Variant = string;
export declare type Fen = string;
export declare type GameNodeId = string;
export declare type Shape = string;
export declare class Pos_ {
    pos: Chess;
    constructor();
    get rules(): Variant;
    get variant(): Variant;
    rawLegalUcis(): string[];
    setVariant(variant: Variant): this;
    setFen(fen: Fen): this;
    reportFen(): string;
    sanToMove(san: San): Move | undefined;
    moveToSan(move: Move): string;
    uciToMove(uci: Uci): Move | undefined;
    moveToUci(move: Move): string;
    play(move: Move): this;
    playSan(san: San): this;
    playUci(uci: Uci): this;
    sanToUci(san: San): Uci | undefined;
    uciToSan(uci: Uci): San | undefined;
    toString(): string;
    legalsForUci(uci: Uci): string[];
    isUciLegal(uci: Uci): boolean;
    isSanLegal(san: San): boolean;
    allLegalUcis(): string[];
    allLegalSans(): (string | undefined)[];
}
export declare function Pos(): Pos_;
declare type RichLegalSan = {
    san: San | undefined;
    uci: Uci | undefined;
    rating: string;
    class: string;
    isMainline: boolean;
    isVariation: boolean;
    weights: number[];
};
export declare class Game_ {
    variant: Variant;
    pos: Pos_;
    root: GameNode_;
    current: GameNode_;
    nodes: {
        [id: string]: GameNode_;
    };
    constructor();
    nodeForSan(san: San, create: boolean): GameNode_ | undefined;
    childNodes(): GameNode_[];
    richLegalSans(): RichLegalSan[];
    fromProps(props: any): this;
    serialize(): any;
    stringify(): string;
    parse(json: string): Game_;
    reportFen(): Fen;
    prev(): GameNode_;
    next(): GameNode_;
    setCurrent(node: GameNode_): Game_;
    selectId(id: GameNodeId): Game_;
    back(): boolean;
    toBegin(): boolean;
    forward(): boolean;
    toEnd(): boolean;
    del(): boolean;
    mainLine(): GameNode_[];
    setVariant(variant: Variant, fen: Fen | undefined): Game_;
    playSans(sans: San[]): Game_;
    playSansStr(sans: string): Game_;
    playSan(san: San): boolean;
    playUci(uci: Uci): boolean;
    clone(): Game_;
}
export declare function Game(): Game_;
export declare class GameNode_ {
    id: GameNodeId;
    fen: Fen;
    genUci: Uci;
    genSan: San;
    weights: number[];
    comment: string;
    shapes: Shape[];
    childs: GameNodeId[];
    index: number;
    parentGame: Game_;
    constructor(parentGame: Game_);
    clone(): GameNode_;
    fromProps(props: any): this;
    serialize(): any;
    stringify(): string;
    parse(json: string): GameNode_;
    prevId(): GameNodeId;
    addChild(childId: GameNodeId): GameNode_;
    removeChild(childId: GameNodeId): GameNode_;
    bringForwardChild(childId: GameNodeId): GameNode_;
    get isTerminal(): boolean;
    getNode(id: GameNodeId): GameNode_;
    next(): GameNode_;
    get isRoot(): boolean;
    prev(): GameNode_;
    get displaySan(): San;
    childNodes(): GameNode_[];
}
export declare function GameNode(parentGame: Game_): GameNode_;
